class ComponentsController < ApplicationController
  model_data_store(Component, {create: true, read: true, update: true, delete: true, index: true}, {create: true, read: true, update: true, delete: true})
  before_action :set_target, only: [:logger_config, :show, :update, :upgrade, :destroy, :endpoint, :stop, :delegate, :restart]

  # POST /components
  # POST /components.json
  def create
    @model = Component.new(model_params)

    if @model.save
      @model = @model.becomes(@model.type.constantize)
      render json: @model, status: :created
      @model.ping
    else
      render json: @model.errors, status: :unprocessable_entity
    end
  end

  # PATCH/PUT /components/1
  # PATCH/PUT /components/1.json
  def update
    if @model.update(model_params)
      @model = @model.becomes(@model.type.constantize)
      render json: @model
      @model.ping
    else
      render json: @model.errors, status: :unprocessable_entity
    end
  end

  def stop
    response = RxModule::Communicator.call_module_method(@model&.module_id, "/api/rx_module/stop", :put)
    if response.is_a? String
      render json: {status: :not_found}, status: :not_found
    else
      render json: response
    end
  end

  def restart
    response = RxModule::Communicator.call_module_method(@model&.module_id, "/api/rx_module/restart", :put)
    if response.is_a? String
      render json: {status: :not_found}, status: :not_found
    else
      render json: response
    end
  end

  def upgrade
    if LicenceInfo.can_download_updates?
      component = @model.becomes(@model.type.constantize)
      Rails.logger.info { {msg: "Module upgrade requested", component: component} }
      component.upgrade!
      Rails.logger.info { {msg: "Module upgrade in progress", component: component} }
      render json: {error: false}
    else
      render json: {error: true, error_details: "Your licence does not allow you to download updates. Please contact support"}, status: 403
    end
  rescue => err
    Rails.logger.error { {msg: "Module upgrade failed", component: component} }
    render json: {error: true, error_details: err}, status: 500
  end

  def endpoint
    render json: {
      url: @model.resolve_endpoint(params[:target]),
      external_url: @model.resolve_endpoint_external(params[:target]),
      authorization: @model.endpoint_authorization(params[:target]),
      delegate: @model.delegate_requests_through_hub?
    }
  end

  def startup
    params.permit!
    Rails.logger.info { "Module startup on #{request.remote_ip}:#{params[:port]}" }
    ip_address = request.remote_ip
    if %w[127.0.0.1 localhost].include?(ip_address) && (hub_ip = Component.ensure_self.ip_address)
      ip_address = hub_ip
      Rails.logger.info { "Mapping #{request.remote_ip} to #{ip_address}" }
    end
    @model = RxModuleComponent.where(module_id: params[:id]).eager_load(:component_permissions).first_or_create
    @model.update(ip_address: ip_address, port: params[:port], reachable: true, is_updating: false)
    @model.process_ping_result(params.to_hash.with_indifferent_access)

    @model.save if @model.changed?

    render json: @model
  end

  def rxuniverse_api_startup
    api_component = RxUniverseApiComponent.first
    Rails.logger.info { {msg: "RxU API upgrade - restart complete", component: api_component} }
    sleep(1)
    api_component.ping # this is only called when an update is complete
  end

  def register_api_component
    params.permit!
    Rails.logger.info { "Module startup on #{request.remote_ip}:#{params[:port]}" }
    ip_address = request.remote_ip
    if %w[127.0.0.1 localhost].include?(ip_address) && (hub_ip = Component.ensure_self.ip_address)
      ip_address = hub_ip
      Rails.logger.info { "Mapping #{request.remote_ip} to #{ip_address}" }
    end
    @model = RouterApiComponent.where(module_id: params[:id]).first_or_create
    @model.update(ip_address: ip_address, port: params[:port])
    render json: @model
  end

  def email_enabled
    component = Component.find_by module_id: params[:id]
    if component.nil?
      render json: {enabled: false, component: nil}
    elsif component[:email_id]
      render json: {enabled: true, is_default: false}
    else
      default_email = Email.find_by is_system_default: true
      if default_email
        render json: {enabled: true, is_default: true}
      else
        render json: {enabled: false}
      end
    end
  end

  def icon
    component = Component.resolve(params[:component_id])
    if component&.icon_file&.attached?
      response.headers["Content-Type"] = component.icon_file.content_type
      component.icon_file.download do |chunk|
        response.stream.write(chunk)
      end
    else
      head :no_content
    end
  end

  def send_email
    component = Component.find_by module_id: params[:id]
    if component.nil?
      render json: {success: false, component: nil}
    else
      email = if component[:email_id]
        component.email
      else
        Email.find_by is_system_default: true
      end

      if email.nil?
        render json: {success: false, email_setup: false}
      else
        begin
          email.send_email(email_params[:to], email_params[:subject], email_params[:body], email_params[:attachments], email_params[:sync])
          render json: {success: true}
        rescue => e
          Rails.logger.error({msg: "Email failed to send", component: component, error: e})
          render json: {success: false, error: e.message}
        end
      end
    end
  end

  def update_icon
    @model = Component.resolve(params[:component_id])
    if @model
      @model.update_icon
      render "components/show"
    else
      head :no_content
    end
  end

  private

  # Use callbacks to share common setup or constraints between actions.
  def set_target
    id = params[:id]
    id = "rx_universe_api" if id == "rxuniverse_api"
    @model = Component.resolve(id)
    raise ActiveRecord::RecordNotFound unless @model
  end

  def before_set_targets
    if !params[:sorted] || (params[:sorted].length == 0)
      @models.order(:module_name)
    else
      @models
    end
  end

  def applicable_authorization
    # we suppress the read check so other modules can call this.  We maintain the read permission for frontend purposes though.
    auth = super
    if auth && auth[:id].to_s == "read"
      false
    else
      auth
    end
  end

  def model_params
    comp_params = params.require(:component).permit(:ip_address, :delegate_requests_through_hub, :external_address, :external_port, :port, :type, :email_id, :protocol)
    comp_params.delete(:type) if comp_params.has_key?(:type) && !%w[RxModuleComponent RxUniverseApiComponent MesApiComponent RouterApiComponent].include?(comp_params[:type])
    comp_params
  end

  def email_params
    e_p = params.permit(:subject, :body, :sync, to: [], attachments: [:name, :file, :path])
    attachments_dir = Rails.root.join("storage", "attachments")
    FileUtils.mkdir_p(attachments_dir)
    e_p[:attachments] = e_p[:attachments]&.map do |a|
      dest = File.join(attachments_dir, a[:name])
      FileUtils.cp a[:file].tempfile, dest
      {
        name: a[:name],
        path: dest
      }
    end
    e_p[:attachments]&.each { |a| a.permit! }
    if e_p[:sync].is_a?(String)
      e_p[:sync] = (e_p[:sync]&.downcase == "true")
    end
    e_p
  end
end
