import {attribute, Model, model, Transport, Types} from '@rx-suite/data';
import * as yup from 'yup';
import {TestContext} from 'yup';
import {t} from "@lingui/macro";
import {action, computed, observable, runInAction} from "mobx";
import semver from 'semver';
import {RxModulePackageModel, RxModulePackageStore} from "./RxModulePackageStore";
import {LoggerConfigStore} from "./LoggerConfigStore";
import {Moment} from 'moment';

@model({
    transportLayer: new Transport("/api/components", "component"),
    permissionTarget: "Component",
    routingInfo: {
        list: '/apps?tab=installed',
        create: '/apps/create',
        read: '/apps/:id/read',
        update: '/apps/:id/edit'
    },
})
export class RxModuleComponentModel extends Model {
    static types: Types = {
        last_ping_at: "date",
        last_successful_ping_at: "date",
        last_ping_result: "json"
    }
    @observable
    loggerConfigState = "missing";

    @attribute({
        label: t`Type`,
        description: t`Rx-Hub will select an appropriate communication protocol based on the API type`,
        validation: yup.string().required(t`Type is a required field`)
    })
    type = 'RxModuleComponent';

    @attribute({autoForm: false, autoTable: false})
    icon_url = "";

    @attribute({
        label: t`IP Address or Network Identifier`,
        description: t`This can be a DNS name or an IP address`,
        validation: yup.string().when(['port', 'protocol'], function whenHandler(port: string, protocol: string, schema: yup.StringSchema) {
            return schema.test('valid-url-combo', t`Invalid URL`, function validateURL(this: TestContext, value: string) {
                const urlString = `${protocol}://${value}:${port}`;
                try {
                    new URL(urlString);
                    return true;
                } catch {
                    return this.createError({message: t`Protocol, Network ID, and Port must form a valid url (result: : ${urlString})`})
                }
            })
        })
    })
    ip_address = '';

    @attribute({
        label: t`Port`,
        description: t`The port the app is listening on`,
        validation: yup.number().typeError(t`Port must be a number`).min(0, t`Port must be a number greater than 0`).max(65535, t`Port cannot exceed 65535`)
    })
    port = 3000;

    @attribute({
        label: t`Name`,
        validation: yup.string().ensure(),
        autoForm: false
    })
    module_name = '';

    @attribute({
        label: t`Module ID`,
        validation: yup.string().ensure(),
        autoForm: false
    })
    module_id = '';

    @attribute({
        label: t`Module Description`,
        validation: yup.string().ensure(),
        autoForm: false
    })
    description = '';

    @attribute({
        label: t`Module Version`,
        validation: yup.string().nullable(),
        autoForm: false
    })
    version = '';

    @attribute({
        label: t`Reachable`,
        validation: yup.boolean().nullable(),
        autoForm: false
    })
    reachable = false;

    @attribute({
        label: t`Updating`,
        validation: yup.boolean().nullable(),
        autoForm: false
    })
    is_updating = false;

    @attribute({
        label: t`Email Address`,
        validation: yup.number().nullable()
    })
    email_id: string | null = null

    @attribute({
        label: t`Last Update Error`,
        validation: yup.string().nullable(),
        autoForm: false
    })
    last_update_error = '';

    @attribute({
        label: t`Update Status`,
        validation: yup.string().nullable(),
        autoForm: false
    })
    update_status = '';

    @attribute({
        label: t`Protocol`,
        description: t`Whether the module uses SSL or not`,
        validation: yup.string().required(t`Protocol is a required field`),
        autoForm: false
    })
    protocol = 'http';

    @attribute({autoForm: false, autoTable: false})
    is_api = '';

    @attribute({
        validation: yup.boolean().nullable()
    })
    has_update_available: boolean | null = true;

    @attribute({
        validation: yup.string().nullable(),
    })
    highest_available_update: string | null = '';

    @attribute()
    licenced = true;

    @attribute({label: t`Last Ping`, autoForm: false, autoTable: false})
    last_ping_at: Moment | undefined = undefined;

    @attribute({label: t`Last Successful Ping`, autoForm: false, autoTable: false})
    last_successful_ping_at: Moment | undefined = undefined;

    @attribute({label: t`Last Ping Result`, autoForm: false, autoTable: false})
    last_ping_result: Record<string, unknown> = {};

    @attribute({ label: t`External Address`, autoTable: false })
    external_address = "";

    @attribute({ label: t`External Port`, autoTable: false })
    external_port = 80;

    @attribute({ label: t`Delegate Requests Through Hub?`, autoTable: false })
    delegate_requests_through_hub = false;

    constructor() {
        super();
        super.setup();
    }

    @action
    async update() {
        return (await (await this.store.transportLayer.fetch(`/${this.module_id}/update`)).json()) as { error: boolean, error_details?: string };
    }

    @action
    async stop() {
        return this.store.transportLayer.fetch(`/${this.module_id}/stop`, {method: "PUT"});
    }

    @action
    async restart() {
        return this.store.transportLayer.fetch(`/${this.module_id}/restart`, {method: "PUT"});
    }

    @computed
    get url() {
        return `${this.protocol}://${this.ip_address}:${this.port}`
    }

    @action
    async fetchLoggerConfig() {
        this.loggerConfigState = "fetching";
        const resp = await this.store.transportLayer.fetch(`/${this.module_id}/logger_config`);
        try {
            const json = await resp.json();
            console.log(json);
            runInAction(() => {
                const model = LoggerConfigStore.createModel(json);
                model.module_id = this.module_id;
                LoggerConfigStore.setModel(model);
                LoggerConfigStore.selectModel(model.getId());
                this.loggerConfigState = "present";
            })
        } catch (e) {
            runInAction(() => {
                this.loggerConfigState = "none";
            })
        }
    }

    get loggerConfig() {
        if (this.loggerConfigState === "missing") {
            this.fetchLoggerConfig();
            return null;
        } else if (this.loggerConfigState === "fetching") {
            return null;
        } else if (this.loggerConfigState === "none") {
            return null;
        } else {
            return LoggerConfigStore.selectedModel;
        }
    }

    @computed
    get updateText() {
        if (this.is_updating) {
            return this.update_status || t`Pending`
        } else if (this.last_update_error) {
            return this.last_update_error
        } else {
            return t`No Error`;
        }
    }

    fetchUpdateData = () => {
        return RxModulePackageStore.get(this.module_id);
    }

    @computed
    get highestAvailableUpdate() {
        const updateInfo = RxModulePackageStore.data.get(this.module_id) as RxModulePackageModel | undefined;
        if (!updateInfo) {
            return false;
        }
        return updateInfo.highestAvailableUpdate;
    }

    @computed
    get updateAvailable() {
        const highestVersion = this.highestAvailableUpdate;
        if (!highestVersion) {
            return false;
        } else if (!this.version) {
            return true;
        } else {
            // If `this.version` is an internal release, the semver conversion will fail
            // If we're on an internal build, we're outside of the update system anyway, so return false
            try {
                return semver.gt(highestVersion, this.version);
            } catch (e) {
                console.error(e);
                console.error(`The above error occurred for module ${this.module_name}`)
                return false;
            }

        }
    }


    @computed get isRxModule() {
        return this.type === 'RxModuleComponent'
    }

    @computed get usesIcon() {
        return this.isRxModule && !this.is_api;
    }
}

export const RxModuleComponentStore = RxModuleComponentModel.store;