# == Schema Information
#
# Table name: users
#
#  id                   :bigint           not null, primary key
#  email                :string
#  name                 :string
#  password_digest      :string
#  password_reset_token :string
#  table_size           :integer          default(10)
#  created_at           :datetime         not null
#  updated_at           :datetime         not null
#  role_id              :bigint
#
# Indexes
#
#  index_users_on_email    (email)
#  index_users_on_role_id  (role_id)
#
# Foreign Keys
#
#  fk_rails_...  (role_id => roles.id) ON DELETE => cascade
#
class User < ApplicationRecord
  has_secure_password

  validate :validate_duplicate_username
  validate :validate_duplicate_email

  belongs_to :role
  has_many :login_tokens
  has_many :o_auths

  after_save do
    InvalidateUserCachesJob.perform_later
  end

  class << self
    def get_by_login_id(login_id)
      where("email IS NOT NULL AND lower(email) = ?", login_id.downcase).or(where("name IS NOT NULL AND lower(name) = ?", login_id.downcase)).first
    end

    def for_email_picker
      includes(:o_auths).all.map do |user|
        {
          id: user[:id],
          name: user[:name],
          email: user[:email],
          rxu: !!user.rxuniverse_service
        }
      end
    end

    def secret_attributes
      %i[password password_digest password_reset_token]
    end
  end

  def validate_duplicate_username
    scope = User.where.not(id: id).where("email IS NOT NULL AND lower(email) = ?", name.downcase).or(User.where.not(id: id).where("name IS NOT NULL AND lower(name) = ?", name.downcase))
    if scope.count > 0
      errors.add(:name, "duplicates another #{scope.first.email && (scope.first.email.downcase == name.downcase) ? "email" : "username"} in the system (case-insensitive)")
    end
  end

  def validate_duplicate_email
    if email
      scope = User.where.not(id: id).where("email IS NOT NULL AND email != '' AND lower(email) = ?", email.downcase).or(User.where.not(id: id).where("name IS NOT NULL AND lower(name) = ?", email.downcase))
      if scope.count > 0
        errors.add(:email, "duplicates another #{scope.first.email && (scope.first.email.downcase == email&.downcase) ? "email" : "username"} in the system (case-insensitive)")
      end
    end
  end

  def generate_random_string
    Digest::SHA1.hexdigest([Time.now, rand].join)
  end

  def rxuniverse_service
    o_auths.where(selected: true).find_by(service: :rxuniverse)
  end

  def generate_token(routine = nil)
    token = generate_random_string
    login_tokens.create(value: token, routine: routine)
    token
  end

  def send_password_reset_email
    RxModule::Communicator::Emails.send_email(email, "Rx-Suite Password Recovery", "You're receiving this email because someone has requested a new password for Rx-Suite
<br/>
Please click here to continue changing your password: #{token_link!}")
  end

  def generate_recovery_token
    update(password_reset_token: generate_random_string)
    password_reset_token
  end

  def token_link(token)
    RxModule::Communicator.resolve_endpoint_external(RxModule.module_id, "/password_reset/#{id}/#{token}")
  end

  def token_link!
    RxModule::Communicator.resolve_endpoint_external(RxModule.module_id, "/password_reset/#{id}/#{generate_recovery_token}")
  end

  def remove_recovery_token
    update(password_reset_token: nil)
  end

  def consume_recovery_token(token)
    if password_reset_token && (password_reset_token == token)
      remove_recovery_token
      true
    else
      false
    end
  end

  def json_merged_options(original_options)
    other_options = {except: User.secret_attributes}
    if original_options&.[](:except)
      if original_options[:except].is_a?(Array)
        other_options[:except].concat(original_options[:except])
      else
        other_options[:except].push(original_options[:except])
      end
    end
    original_options.merge(other_options)
  end

  def to_json(options = {})
    super(json_merged_options(options))
  end

  def as_json(options = {})
    super(json_merged_options(options))
  end
end
